package com.tcctlo.law.controller;

import com.tcctlo.common.core.domain.AjaxResult;
import com.tcctlo.law.entity.IndexStatistics;
import com.tcctlo.law.searcher.TimeSearcher;
import com.tcctlo.law.service.IIndexService;
import com.tcctlo.law.vo.IndexLawCaseMoneyInfo;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 首页控制器
 */
@RestController
@RequestMapping("/caseIndex")
public class IndexController {

    @Resource
    private IIndexService iIndexService;

    /**
     * 案件类型统计  conflictStatistics
     *
     * @return 结果集
     */
    @GetMapping("/caseTypeStatistics")
    public AjaxResult caseTypeStatistics(@RequestParam(value = "startTime", required = false) @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
                                         @RequestParam(value = "endTime", required = false) @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date endTime) {
        List<IndexStatistics> integerIntegerMap = iIndexService.caseTypeStatistics(startTime, endTime);
        return AjaxResult.success(integerIntegerMap);
    }

    /**
     * 利冲统计
     *
     * @return 结果集
     */
    @GetMapping("/conflictStatistics")
    public AjaxResult conflictStatistics(@RequestParam("startTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
                                         @RequestParam("endTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date endTime) {
        List<IndexStatistics> integerIntegerMap = iIndexService.conflictStatistics(startTime, endTime);
        return AjaxResult.success(integerIntegerMap);
    }

    /**
     * 费用统计
     *
     * @return 结果集
     */
    @GetMapping("/costStatistics")
    public AjaxResult costStatistics(@RequestParam("startTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
                                     @RequestParam("endTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date endTime) {
        List<IndexStatistics> integerIntegerMap = iIndexService.costStatistics(startTime, endTime);
        return AjaxResult.success(integerIntegerMap);
    }

    /**
     * 收益走势
     *
     * @return 结果集
     */
    @GetMapping("/earningsTrend")
    public AjaxResult earningsTrend(@RequestParam("startTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date startTime,
                                    @RequestParam("endTime") @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss") Date endTime) {
        List<IndexStatistics> integerIntegerMap = iIndexService.earningsTrend(startTime, endTime);
        return AjaxResult.success(integerIntegerMap);
    }

    /**
     * 统计所有的律师信息
     * @return
     */
    @PostMapping("/lawInfo")
    public AjaxResult lawInfo(TimeSearcher searcher) {
        List<IndexLawCaseMoneyInfo> result = iIndexService.lawInfo(searcher);
        return AjaxResult.success(result);
    }

}
