package com.tcctlo.law.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctlo.common.core.domain.AjaxResult;
import com.tcctlo.law.entity.InvoiceRecord;
import com.tcctlo.law.vo.InvoiceRecordVO;
import com.tcctlo.law.service.IInvoiceRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 开票记录表 前端控制器
 * </p>
 *
 * @author
 * @since 2025-03-07
 */

@RestController
@RequestMapping("/invoiceRecord")
public class InvoiceRecordController {


    @Autowired
    private IInvoiceRecordService iInvoiceRecordService;

    @GetMapping(value = "/list")
    public AjaxResult list(@RequestParam(required = false) Integer current, @RequestParam(required = false) Integer size) {
        if (current == null) {
            current = 1;
        }
        if (size == null) {
            size = 10;
        }
        Page<InvoiceRecord> pageList = iInvoiceRecordService.list(current, size);
        return AjaxResult.success(pageList);
    }

    /**
     * 根据案件ID获取开票记录
     * @param caseId 案件ID
     * @return 结果集
     */
    @GetMapping(value = "/selectByCaseId/{caseId}")
    public AjaxResult selectByCaseId(@PathVariable("caseId") Long caseId) {
        List<InvoiceRecordVO> res = iInvoiceRecordService.selectByCaseId(caseId);
        return AjaxResult.success(res);
    }


    @GetMapping(value = "/{id}")
    public AjaxResult getById(@PathVariable("id") Long id) {
        InvoiceRecord invoiceRecord = iInvoiceRecordService.getById(id);
        return AjaxResult.success(invoiceRecord);
    }

    @PostMapping(value = "/create")
    public AjaxResult create(@RequestBody InvoiceRecordVO params) {
        return AjaxResult.success(iInvoiceRecordService.create(params));
    }

    @PostMapping(value = "/delete/{id}")
    public AjaxResult delete(@PathVariable("id") Long id) {
        Integer result = iInvoiceRecordService.delete(id);
        return AjaxResult.success(result);
    }

    /**
     * 逻辑删除
     * @param params 参数
     * @return 结果集
     */
    @PostMapping(value = "/deleteBatch")
    public AjaxResult deleteBatch(@RequestBody Map<String, String> params) {
        AjaxResult ajaxResult = new AjaxResult();
        Integer result = iInvoiceRecordService.deleteBatch(params);
        if (result > 0) {
            ajaxResult.put("msg", "删除成功");
            ajaxResult.put("code", 200);
        } else {
            ajaxResult.put("msg", "删除失败");
            ajaxResult.put("code", 500);
        }
        return ajaxResult;
    }

    @PostMapping(value = "/update")
    public AjaxResult update(@RequestBody InvoiceRecord params) {
        Integer result = iInvoiceRecordService.update(params);
        return AjaxResult.success(result);
    }
}
