package com.tcctlo.law.entity;

import java.math.BigDecimal;

import com.baomidou.mybatisplus.annotation.*;

import java.util.Date;
import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.JsonNode;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * <p>
 * 案件信息表
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName(value = "case_information", autoResultMap = true)
public class CaseInformation implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 案件编号【自动生成UUID】
     */
    private String caseNo;

    /**
     * 案件名称
     */
    private String caseName;

    /**
     * 案件内容
     */
    private String caseContext;

    /**
     * 业务类型【0：诉讼业务，1：非诉讼业务】
     */
    private String businessType;

    /**
     * 业务分类【0：民商事业务、1：刑事辩护业务、2：行政复议及诉讼业务、3：企业常年法律顾问、4：企业专项法律顾问】
     */
    private String businessClassify;

    /**
     * 案件类型
     */
    private String caseType;

    /**
     * 代理阶段
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private JsonNode agencyStage;

    /**
     * 案件来源【0：市场案源，1：律师独立案源】
     */
    private String caseSource;

    /**
     * 应收金额
     */
    private BigDecimal amountReceivable;

    /**
     * 已收金额总计
     */
    private BigDecimal amountReceivedSum;

    /**
     * 付款期限
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date paymentDeadline;

    /**
     * 付款期限备注
     */
    private String paymentDeadlineRemark;

    /**
     * 预留款
     */
    private BigDecimal reserveMoney;

    /**
     * 顾问期限开始时间
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date termOfConsultancyStart;

    /**
     * 顾问期限结束时间
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date termOfConsultancyEnd;

    /**
     * 收款状态【0：未付款、1：部分付款，2：已结清】
     */
    private Integer collectionStatus;

    /**
     * 利冲状态【0：未检索，1：疑似利冲，2：检索通过】
     */
    private Integer impulseState;

    /**
     * 收案状态【0：利冲收案、1：正常收案、2：终止收案】
     */
    private Integer caseStatus;

    /**
     * 审核状态【0：未审核，1：已审核，2：审核不通过】
     */
    private Integer auditStatus;

    /**
     * 归档状态【0：未归档，1：已归档】
     */
    private Integer archiveStatus;

    /**
     * 是否结案【0：未结案，1：已结案】
     */
    private Integer isFinishCase;

    /**
     * 审核意见
     */
    /*private String auditOpinion;*/

    /**
     * 案件涉及委托方/相对方ID集合
     */
    private String impulseInformationIds;

    /**
     * 案件涉及律师ID集合
     * FIXME 关联的地方进行修改后，将该字段删除
     */
    private String caseLawyerIds;

    /**
     * 委托律师
     */
    private Long entrustedLawyerId;

    /**
     * 协办律师，多个逗号分隔
     */
    private String coLawyerIds;

    /**
     * 案件信息备注
     */
    private String caseRemark;

    /**
     * 状态【0：正常，1：停用】
     */
    private Integer status;

    /**
     * 删除标志【0：未删除，1：已删除】
     */
    @TableLogic
    private Integer delFlag;

    /**
     * 创建者
     */
    private String createBy;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新者
     */
    private String updateBy;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 备注
     */
    private String remark;

    /**
     * 委托律师姓名
     */
    @TableField(exist = false)
    private String entrustedLawyerName;

    /**
     * 协办律师姓名,多个英文逗号分隔
     */
    @TableField(exist = false)
    private String coLawyerNames;

}
