package com.tcctlo.law.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.law.entity.CaseInformation;
import com.tcctlo.law.entity.CaseTransfer;
import com.tcctlo.law.entity.TransferParam;
import com.tcctlo.law.mapper.CaseInformationMapper;
import com.tcctlo.law.mapper.CaseTransferMapper;
import com.tcctlo.law.service.ICaseTransferService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 案件律师表 服务实现类
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Service
public class CaseTransferServiceImpl extends ServiceImpl<CaseTransferMapper, CaseTransfer> implements ICaseTransferService {


    @Resource
    private CaseTransferMapper transferMapper;

    @Resource
    private CaseInformationMapper caseInformationMapper;


    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    @Deprecated
    public boolean transferLawyer(TransferParam params) {
        try {
            // 获取新的主板律师ID
            Long new_entrusted_lawyer_id = params.getNewLawyerId();
            //获取要转移的案件IDS
            List<Long> caseIdList = StrUtil.split(params.getCaseIds(), ",").stream().map(Long::parseLong).collect(Collectors.toList()); //获取要转移的案件ID

            caseInformationMapper.selectList(new QueryWrapper<CaseInformation>().in("id", caseIdList)).forEach(item -> {
                //获取案件信息表中的【案件涉及律师ID】
                Long old_entrusted_lawyer_id = item.getEntrustedLawyerId();
                // 就案件信息写入caseLawyer表
                CaseTransfer caseTransfer = new CaseTransfer();
                caseTransfer.setCaseId(item.getId());
                caseTransfer.setLawyerId(old_entrusted_lawyer_id);
                caseTransfer.setDelFlag(0);
                // 查看是否是原律师，不存在则为原律师,插入旧数据，新数据不变
                if (Math.toIntExact(transferMapper.selectCount(new QueryWrapper<CaseTransfer>().eq("case_id", item.getId()).eq("lawyer_id", old_entrusted_lawyer_id))) > 0) {
                    caseTransfer.setIsPrime(1);
                } else {
                    caseTransfer.setIsPrime(0);
                }
                caseTransfer.setCreateTime(new Date());
                caseTransfer.setCreateBy(SecurityUtils.getLoginUser().getUserId().toString());
                transferMapper.insert(caseTransfer);

                // 更新案件信息表中的【案件涉及律师ID】
                item.setEntrustedLawyerId(new_entrusted_lawyer_id);
                item.setUpdateTime(new Date());
                item.setUpdateBy(SecurityUtils.getLoginUser().getUserId().toString());
                caseInformationMapper.updateById(item);
            });
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("转接失败");
        }
    }

}
