package com.tcctlo.law.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.core.domain.entity.SysRole;
import com.tcctlo.common.core.domain.model.LoginUser;
import com.tcctlo.common.utils.SecurityUtils;
import com.tcctlo.law.entity.CollectionRecord;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctlo.law.entity.FileManager;
import com.tcctlo.law.mapper.CaseInformationMapper;
import com.tcctlo.law.mapper.CollectionRecordMapper;
import com.tcctlo.law.mapper.FileManagerMapper;
import com.tcctlo.law.service.ICollectionRecordService;
import com.tcctlo.law.service.IFileManagerService;
import com.tcctlo.law.vo.CaseInformationListVO;
import com.tcctlo.law.vo.CollectionRecordEnterVO;
import com.tcctlo.system.mapper.SysUserMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 收款记录表 服务实现类
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Service
public class CollectionRecordServiceImpl extends ServiceImpl<CollectionRecordMapper, CollectionRecord> implements ICollectionRecordService {


    @Resource
    private CollectionRecordMapper collectionRecordMapper;

    @Resource
    private IFileManagerService fileManagerService;

    @Resource
    private FileManagerMapper fileManagerMapper;

    @Resource
    private CaseInformationMapper caseInformationMapper;

    /**
     * 系统用户mapper
     */
    @Resource
    private SysUserMapper sysUserMapper;


    @Override
    public Page<CollectionRecord> list(Integer pageNo, Integer pageSize) {
        return collectionRecordMapper.selectPage(new Page<>(pageNo, pageSize), new QueryWrapper<>());
    }

    @Override
    public CollectionRecord getById(Long id) {
        CollectionRecord collectionRecord = collectionRecordMapper.selectById(id);
        return collectionRecord;
    }

    @Override
    public Integer create(CollectionRecord collectionRecord) {
        return collectionRecordMapper.insert(collectionRecord);
    }

    @Override
    public Integer delete(Long id) {
        return collectionRecordMapper.deleteById(id);
    }

    @Override
    public Integer update(CollectionRecord collectionRecord) {
        return collectionRecordMapper.updateById(collectionRecord);
    }


//    @Override
//    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
//    public void casePayment(CollectionRecordEnterVO params) {
//        try {
//            LoginUser loginUser = SecurityUtils.getLoginUser();
//            Long userId = loginUser.getUserId();
//            String nickName = loginUser.getUser().getNickName();
//            CollectionRecord collectionRecord = BeanUtil.copyProperties(params, CollectionRecord.class);
//            collectionRecord.setUserId(userId);
//            collectionRecord.setUserName(nickName);
//            String collect = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.joining(","));
//            if (!collect.contains("FINANCE")) {
//                collectionRecord.setAuditStatus(0);
//            } else {
//                collectionRecord.setAuditStatus(1);
//            }
//
//            List<FileManager> fileManagerList = params.getFileManager().stream().peek(item -> item.setFileType(1)).collect(Collectors.toList());
//            fileManagerService.saveBatch(fileManagerList);
//            String fileIds = fileManagerList.stream().map(fileManager -> fileManager.getId().toString()).collect(Collectors.joining(","));
//            collectionRecord.setCollectionUrlId(fileIds);
//            collectionRecordMapper.insert(collectionRecord);
//
//
//            //修改案件信息表中的已收金额-----根据案件ID查询缴费记录，做一个累加
//            List<CollectionRecord> lists = BeanUtil.copyToList(collectionRecordMapper.selectByCaseId(params.getCaseId()), CollectionRecord.class);
//            BigDecimal reduce = lists.stream().map(CollectionRecord::getAmountReceived).reduce(BigDecimal.ZERO, BigDecimal::add);
//            caseInformationMapper.updateCaseAmount(params.getCaseId(), reduce); //修改案件信息表中已收金额总计
//
//        } catch (Exception e) {
//            e.printStackTrace();
//            throw new RuntimeException("缴费失败");
//        }
//    }

//    @Override
//    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
//    public Boolean casePaymentBatch(List<CollectionRecordEnterVO> params) {
//        try {
//            LoginUser loginUser = SecurityUtils.getLoginUser();
//            Long userId = loginUser.getUserId();
//            String nickName = loginUser.getUser().getNickName();
//
//            //TODO  待调整
//            if (CollUtil.isNotEmpty(params)) {
//                List<CollectionRecordEnterVO> newData = params;  //收款记录信息
//
//                //获取新添加的收款记录，计算出新的收款记录金额，在总金额中加上
//                List<CollectionRecordEnterVO> insert = newData.stream().filter(item -> item.getId() == null).collect(Collectors.toList());
//                insert.forEach(item -> {
//                    if (CollUtil.isEmpty(item.getFileManager())){
//                        throw new RuntimeException("请上传缴费凭证");
//                    }
//                    List<FileManager> fileManager = item.getFileManager().stream().peek(item1 -> item1.setFileType(1)).collect(Collectors.toList());
//                    fileManagerService.saveBatch(fileManager);
//                    item.setCollectionUrlId(fileManager.stream().map(i -> i.getId().toString()).collect(Collectors.joining(",")));
//                    item.setUserId(userId);
//                    item.setUserName(nickName);
//                    String collect = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.joining(","));
//                    if (!collect.contains("CAIWU")) {
//                        item.setAuditStatus(0);
//                    } else {
//                        item.setAuditStatus(1);
//                    }
//                });
//                List<CollectionRecord> list = BeanUtil.copyToList(insert, CollectionRecord.class);
//                BigDecimal addMoney = list.stream().map(CollectionRecord::getAmountReceived).reduce(BigDecimal.ZERO, BigDecimal::add);    //新增收款记录金额
//                this.saveBatch(list);
//                Long caseId = list.get(0).getCaseId();
//                CaseInformationListVO caseInfo = caseInformationMapper.selectCaseById(caseId);
//                caseInformationMapper.updateCaseAmount(caseId, caseInfo.getAmountReceivedSum().add(addMoney));
//                CaseInformationListVO updateInfo = caseInformationMapper.selectCaseById(caseId);
//                Date paymentDeadline = updateInfo.getPaymentDeadline();
//                if (paymentDeadline != null) {
//                    if (updateInfo.getAmountReceivedSum().compareTo(new BigDecimal(0.0)) == 0){
//                        caseInformationMapper.updateCollectionStatus(updateInfo.getId(),0);
//                    }
//                    if (updateInfo.getAmountReceivedSum().compareTo(updateInfo.getAmountReceivable()) < 0){
//                        caseInformationMapper.updateCollectionStatus(updateInfo.getId(),1);
//                    }
//                    if (updateInfo.getAmountReceivedSum().compareTo(updateInfo.getAmountReceivable()) == 0){
//                        caseInformationMapper.updateCollectionStatus(updateInfo.getId(),2);
//                    }
//
//                }
//            }
//            return true;
//        } catch (Exception e) {
//            e.printStackTrace();
//            throw new RuntimeException("批量收款失败");
//        }
//    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Integer deleteLogic(Map<String, Long> params) {
        Long id = params.get("id");
        Long caseId = params.get("caseId");
        CollectionRecord collectionRecord = collectionRecordMapper.selectById(id);
        BigDecimal delMoney = collectionRecord.getAmountReceived(); //要减去的金额
        Integer result = collectionRecordMapper.removeByCaseIdAdnRecordId(id, caseId);   //从收款记录中删除
        caseInformationMapper.updateCaseAmount(caseId, caseInformationMapper.selectCaseById(caseId).getAmountReceivedSum().subtract(delMoney));
        CaseInformationListVO caseInfo = caseInformationMapper.selectCaseById(caseId);
        if (caseInfo.getAmountReceivedSum().compareTo(new BigDecimal(0.0)) == 0) {
            caseInformationMapper.updateCollectionStatus(caseId, 0);
        }
        return result;
    }

    @Override
    public List<CollectionRecordEnterVO> selectByCaseId(Long caseId) {
        List<CollectionRecordEnterVO> lists = collectionRecordMapper.selectByCaseId(caseId);
        for (CollectionRecordEnterVO item : lists) {
            Long userId = item.getUserId();
            String roleKey = sysUserMapper.selectUserRoleByUserId(userId);
            if ("CAIWU".equals(roleKey)){
                item.setIsEdit(item.getAuditStatus() != 1);
            }else {
                item.setIsEdit(item.getAuditStatus() == 0);
            }

        }
        return lists;
    }

    @Override
    public Boolean audit(Map<String, Long> map) {
        Long id = map.get("id");
        return collectionRecordMapper.audit(id) > 0;
    }

    @Override
    @Transactional(isolation = Isolation.DEFAULT, rollbackFor = Exception.class)
    public Integer addCollectionRecord(CollectionRecord collectionRecord) {
        //财务管理--收费--添加收费记录,添加完收费记录后，需要将案件信息里的已收金额进行同步更新
        Long caseId = collectionRecord.getCaseId();
        BigDecimal addMoney = collectionRecord.getAmountReceived();
        LoginUser loginUser = SecurityUtils.getLoginUser();
        Long userId = loginUser.getUserId();
        String nickName = loginUser.getUser().getNickName();
        collectionRecord.setUserName(nickName);
        collectionRecord.setUserId(userId);
        String collect = loginUser.getUser().getRoles().stream().map(SysRole::getRoleKey).collect(Collectors.joining(","));
        if (!collect.contains("CAIWU")) {
            collectionRecord.setAuditStatus(0);
        } else {
            collectionRecord.setAuditStatus(1);
        }
        Integer result = collectionRecordMapper.insert(collectionRecord);
        //更新案件信息里的已收金额
        CaseInformationListVO caseInfo = caseInformationMapper.selectCaseById(caseId);
        caseInformationMapper.updateCaseAmount(caseId, caseInfo.getAmountReceivedSum().add(addMoney));
        return result;
    }

}
