package com.tcctlo.law.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tcctlo.common.core.domain.AjaxResult;
import com.tcctlo.common.utils.StringUtils;
import com.tcctlo.law.entity.*;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tcctlo.law.entity.wordTemplateEntity.ClashMsgCaseInfoItem;
import com.tcctlo.law.mapper.CaseInformationMapper;
import com.tcctlo.law.mapper.ImpulseInformationMapper;
import com.tcctlo.law.service.IImpulseInformationService;
import com.tcctlo.law.vo.CaseInformationListVO;
import com.tcctlo.law.vo.ImpulseInformationVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Consumer;
import java.util.stream.Collectors;

/**
 * <p>
 * 利冲信息表 服务实现类
 * </p>
 *
 * @author 张世琪
 * @since 2025-02-06
 */
@Service
public class ImpulseInformationServiceImpl extends ServiceImpl<ImpulseInformationMapper, ImpulseInformation> implements IImpulseInformationService {


    @Resource
    private ImpulseInformationMapper impulseInformationMapper;

    @Resource
    private CaseInformationMapper caseInformationMapper;

    @Override
    public Page<ImpulseInformationVO> list(Map<String, Object> params) {
        Page<ImpulseInformationVO> resPage = new Page<>();
        Page<ImpulseInformation> page = impulseInformationMapper.selectCondition(new Page<>((Integer) params.get("current"), (Integer) params.get("size")), params);
        if (page.getRecords() != null) {
            List<ImpulseInformationVO> list = BeanUtil.copyToList(page.getRecords(), ImpulseInformationVO.class);
            list.stream().peek(item -> {
                if (item.getCaseId() != null) {
                    CaseInformation caseInformation = caseInformationMapper.selectById(item.getCaseId());
                    item.setCaseInfoName(caseInformation.getCaseName());

                    item.setTermOfConsultancyStart(caseInformation.getTermOfConsultancyStart());
                    item.setTermOfConsultancyEnd(caseInformation.getTermOfConsultancyEnd());

                    item.setArchiveStatus(caseInformation.getArchiveStatus());
                    item.setIsFinishCase(caseInformation.getIsFinishCase());
                }
            }).collect(Collectors.toList());
            resPage.setRecords(list);
            resPage.setTotal(page.getTotal());
            resPage.setSize(page.getSize());
            resPage.setCurrent(page.getCurrent());
        }
        return resPage;
    }

    /*@Override
    public Page<ImpulseInformation> list(Map<String, Object> params) {
        return impulseInformationMapper.selectPage(new Page<>((Integer) params.get("current"), (Integer) params.get("size")), new QueryWrapper<>());
    }*/

    @Override
    public List<ImpulseInformation> selectLikeByParam(String caseName, Integer isClient) {
        LambdaQueryWrapper<ImpulseInformation> wrapper = new LambdaQueryWrapper<>();
        wrapper.like(StringUtils.isNotNull(caseName), ImpulseInformation::getCaseName, caseName)
                .eq(StringUtils.isNotNull(isClient), ImpulseInformation::getIsClient, isClient)
                .eq(ImpulseInformation::getDelFlag, 0)
                .eq(ImpulseInformation::getStatus, 0)
                .orderByDesc(ImpulseInformation::getCreateTime);
        return impulseInformationMapper.selectList(wrapper);
    }

    /*@Override
    public Boolean retrievalConflict(ImpulseInformation impulseInformation) {
        *//**
     * 当 impulseInformation.getIsClient() 的值为 0 时，1 - 0 结果为 1；当 impulseInformation.getIsClient() 的值为 1 时，1 - 1 结果为 0
     * 注意：
     * 1、如果是【委托方】就去检索【相对方】中有没有
     * 2、如果是【相对方】就去检索【委托方】中有没有
     * 3、【委托方】不能是律所中任何一个案件的被告方====【被告方】不能是律所中任何一个案件的【委托方】
     *//*
        impulseInformation.setIsClient(1 - impulseInformation.getIsClient());
        List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(impulseInformation);
        return resList.isEmpty();
    }*/

    /*@Override
    public Map<String, Object> retrievalConflict(ImpulseInformation impulseInformation) {
        */

    /**
     * 当 impulseInformation.getIsClient() 的值为 0 时，1 - 0 结果为 1；当 impulseInformation.getIsClient() 的值为 1 时，1 - 1 结果为 0
     * 注意：
     * 1、如果是【委托方】就去检索【相对方】中有没有
     * 2、如果是【相对方】就去检索【委托方】中有没有
     * 3、【委托方】不能是律所中任何一个案件的被告方====【被告方】不能是律所中任何一个案件的【委托方】
     *//*
        TreeMap<String, Object> resMap = new TreeMap<>();
        Long id = impulseInformation.getId();
        if (id != null){
            resMap.put("result", true);
            resMap.put("clashInfo", null);
            return resMap;
        }

        List<ClashInfoVO> clashInfoVOS = new ArrayList<>();
        *//*impulseInformation.setIsClient(1 - impulseInformation.getIsClient());*//*

        if (impulseInformation.getIsClient() == 0){
            impulseInformation.setIsClient(1);
        }
        if (impulseInformation.getIsClient() == 1){
            impulseInformation.setIsClient(0);
        }

        List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(impulseInformation);
        resMap.put("result", resList.isEmpty());

        for (ImpulseInformation item : resList) {
            ClashInfoVO clashInfoVO = new ClashInfoVO();
            CaseInformationListVO caseInformationListVO = caseInformationMapper.selectCaseById(item.getCaseId());
            clashInfoVO.setCaseId(item.getCaseId());
            clashInfoVO.setCaseInfoName(caseInformationListVO.getCaseName());
            clashInfoVO.setCaseInfoNo(caseInformationListVO.getCaseNo());
            clashInfoVO.setCaseName(item.getCaseName());
            clashInfoVO.setCaseNo(item.getCaseNo());
            clashInfoVO.setCaseType(item.getCaseType());
            clashInfoVO.setIsClient(item.getIsClient());
            clashInfoVOS.add(clashInfoVO);
        }
        resMap.put("clashInfo", clashInfoVOS);
        return resMap;
    }*/
    @Override
    public Map<String, Object> retrievalConflict(ImpulseInformation impulseInformation) {
        /**
         * 当 impulseInformation.getIsClient() 的值为 0 时，1 - 0 结果为 1；当 impulseInformation.getIsClient() 的值为 1 时，1 - 1 结果为 0
         * 注意：
         * 1、如果是【委托方】就去检索【相对方】中有没有
         * 2、如果是【相对方】就去检索【委托方】中有没有
         * 3、【委托方】不能是律所中任何一个案件的被告方====【被告方】不能是律所中任何一个案件的【委托方】
         */
        TreeMap<String, Object> resMap = new TreeMap<>();
        List<ClashInfoVO> clashInfoVOS = new ArrayList<>();
        Long id = impulseInformation.getId();


        if (id != null) {
            ImpulseInformation info = impulseInformationMapper.selectById(id);
            if (info.getCheckFlag() == 0) {
                impulseInformation.setIsClient(1 - impulseInformation.getIsClient());
                List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(impulseInformation);
                resMap.put("result", resList.isEmpty());
                for (ImpulseInformation item : resList) {
                    ClashInfoVO clashInfoVO = new ClashInfoVO();
                    if (Objects.isNull(item.getCaseId())) {
                        clashInfoVO.setCaseId(item.getCaseId());
                        clashInfoVO.setCaseName(item.getCaseName());
                        clashInfoVO.setCaseNo(item.getCaseNo());
                        clashInfoVO.setCaseType(item.getCaseType());
                        clashInfoVO.setIsClient(item.getIsClient());
                    } else {
                        CaseInformationListVO caseInformationListVO = caseInformationMapper.selectCaseById(item.getCaseId());
                        clashInfoVO.setCaseId(item.getCaseId());
                        clashInfoVO.setCaseInfoName(caseInformationListVO.getCaseName());
                        clashInfoVO.setCaseInfoNo(caseInformationListVO.getCaseNo());
                        clashInfoVO.setCaseName(item.getCaseName());
                        clashInfoVO.setCaseNo(item.getCaseNo());
                        clashInfoVO.setCaseType(item.getCaseType());
                        clashInfoVO.setIsClient(item.getIsClient());
                    }
                    clashInfoVOS.add(clashInfoVO);
                }
                resMap.put("clashInfo", clashInfoVOS);
                return resMap;
            } else {
                resMap.put("result", true);
                resMap.put("clashInfo", null);
                return resMap;
            }
        }

        impulseInformation.setIsClient(1 - impulseInformation.getIsClient());
        List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(impulseInformation);
        resMap.put("result", resList.isEmpty());

        for (ImpulseInformation item : resList) {
            ClashInfoVO clashInfoVO = new ClashInfoVO();
            CaseInformationListVO caseInformationListVO = caseInformationMapper.selectCaseById(item.getCaseId());
            if (caseInformationListVO != null) {
                clashInfoVO.setCaseId(ObjUtil.isEmpty(item.getCaseId()) ? null : item.getCaseId());
                clashInfoVO.setCaseInfoName(ObjUtil.isEmpty(caseInformationListVO.getCaseName()) ? null : caseInformationListVO.getCaseName());
                clashInfoVO.setCaseInfoNo(ObjUtil.isEmpty(caseInformationListVO.getCaseNo()) ? null : caseInformationListVO.getCaseNo());
            }


            clashInfoVO.setCaseName(ObjUtil.isEmpty(item.getCaseName()) ? null : item.getCaseName());
            clashInfoVO.setCaseNo(ObjUtil.isEmpty(item.getCaseNo()) ? null : item.getCaseNo());
            clashInfoVO.setCaseType(ObjUtil.isEmpty(item.getCaseType()) ? null : item.getCaseType());
            clashInfoVO.setIsClient(ObjUtil.isEmpty(item.getIsClient()) ? null : item.getIsClient());
            clashInfoVOS.add(clashInfoVO);
        }
        resMap.put("clashInfo", clashInfoVOS);
        return resMap;
    }

    @Override
    public Map<String, Object> retrievalConflictBatch(List<ImpulseInformation> impulseInformation) {
        for (ImpulseInformation info : impulseInformation) {
            info.setIsClient(1 - info.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(info);

        }
        return Collections.emptyMap();
    }

    /*@Override
    public List<String> auditClashMsg(Long caseId) {
        //去利冲信息库查询信息，现将疑似利冲的数据获取出来，去查询这些利冲的数据和哪些案件的数据有利冲
        List<String> resMsg = new ArrayList<>();
        HashMap<ImpulseInformation, List<ImpulseInformation>> map = new HashMap<>();    //如果前端需要
        List<ImpulseInformation> clashInfo = impulseInformationMapper.auditClashMsg(caseId);
        for (ImpulseInformation item : clashInfo) {
            item.setIsClient(1 - item.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(item);
            item.setIsClient(1 - item.getIsClient());
            map.put(item, resList);
            CaseInformation caseItem = caseInformationMapper.selectById(item.getCaseId());
            StringBuilder msg = new StringBuilder("当前案件中：" + (item.getIsClient() == 0 ? "【委托方】->" : "【相对方】->") + item.getCaseName() + "[" + item.getCaseNo() + "]" + "与以下案件中的委托方/相对方有利冲嫌疑：");
            for (ImpulseInformation in : resList) {
                CaseInformation ci = caseInformationMapper.selectById(in.getCaseId());
                //String msg = caseItem.getCaseName() + "【" + caseItem.getCaseNo() + "】案件中的" + (item.getIsClient() == 0 ? "【委托方】" : "【相对方】") + item.getCaseName() + "[" + item.getCaseNo() + "]" + "与" + ci.getCaseName() + "【" + ci.getCaseNo() + "】案件中的" + (in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "[" + in.getCaseNo() + "]" + "有疑似利冲信息";
                msg.append(ci.getCaseName()).append("【").append(ci.getCaseNo()).append("】案件中的").append(in.getIsClient() == 0 ? "【委托方】" : "【相对方】").append(in.getCaseName()).append("[").append(in.getCaseNo()).append("]").append("有疑似利冲信息;");

            }
            resMsg.add(msg.toString());
            msg.setLength(0);
        }
        return resMsg;

    }*/


    /*@Override
    public List<ClashMsg> auditClashMsg(Long caseId) {
        //去利冲信息库查询信息，现将疑似利冲的数据获取出来，去查询这些利冲的数据和哪些案件的数据有利冲
        List<ClashMsg> clashMsgList = new ArrayList<>();
        List<ImpulseInformation> clashInfo = impulseInformationMapper.auditClashMsg(caseId);

        for (ImpulseInformation item : clashInfo) {
            ClashMsg clashMsg = new ClashMsg();
            item.setIsClient(1 - item.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(item);
            item.setIsClient(1 - item.getIsClient());
            CaseInformation caseItem = caseInformationMapper.selectById(item.getCaseId());  //查询当前案件信息【暂时不需要】
            clashMsg.setMsgMain("当前案件中：" + (item.getIsClient() == 0 ? "【委托方】->" : "【相对方】->") + item.getCaseName() + "[" + item.getCaseNo() + "]" + "与以下案件中的委托方/相对方有利冲嫌疑：");
            ArrayList<String> strings = new ArrayList<>();
            for (ImpulseInformation in : resList) {
                CaseInformation ci = caseInformationMapper.selectById(in.getCaseId());
                strings.add(ci.getCaseName() + "【" + ci.getCaseNo() + "】案件中的" + (in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "[" + in.getCaseNo() + "]" + "有疑似利冲信息");
            }
            clashMsg.setMsgSub(strings);
            clashMsgList.add(clashMsg);
        }
        return clashMsgList;

    }*/

    @Override
    public AjaxResult auditClashMsg(Long caseId) {
        AjaxResult ajaxResult = new AjaxResult();
        //去利冲信息库查询信息，现将疑似利冲的数据获取出来，去查询这些利冲的数据和哪些案件的数据有利冲
        List<ClashMsgWithCaseInfo> clashMsgWithCaseInfos = new ArrayList<>();
        List<ImpulseInformation> clashInfo = impulseInformationMapper.auditClashMsg(caseId);
        for (ImpulseInformation item : clashInfo) {
            ClashMsgWithCaseInfo clashMsgWithCaseInfo = new ClashMsgWithCaseInfo();
            item.setIsClient(1 - item.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(item);
            item.setIsClient(1 - item.getIsClient());
            CaseInformation caseItem = caseInformationMapper.selectById(item.getCaseId());  //查询当前案件信息【暂时不需要】
            clashMsgWithCaseInfo.setMsgMain("当前案件中：" + (item.getIsClient() == 0 ? "【委托方】->" : "【相对方】->") + item.getCaseName() + (StringUtils.isEmpty(item.getCaseNo()) ? "" : "[" + item.getCaseNo() + "]") + "与以下案件中的委托方/相对方有利冲嫌疑：");
            List<ClashMsgCaseInfoItem> list = new ArrayList<>();
            for (ImpulseInformation in : resList) {
                CaseInformation ci = null;
                ClashMsgCaseInfoItem clashMsgCaseInfoItem = new ClashMsgCaseInfoItem();
                if (in.getCaseId() != null) {
                    ci = caseInformationMapper.selectById(in.getCaseId());
                    clashMsgCaseInfoItem.setCaseId(ci.getId());
                    clashMsgCaseInfoItem.setMsg(ci.getCaseName() + "【" + ci.getCaseNo() + "】案件中的" + (in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "[" + in.getCaseNo() + "]" + "有疑似利冲信息");
                } else {
                    clashMsgCaseInfoItem.setMsg((in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "有疑似利冲信息");
                }

                list.add(clashMsgCaseInfoItem);
            }
            clashMsgWithCaseInfo.setMsgSub(list);
            clashMsgWithCaseInfos.add(clashMsgWithCaseInfo);
        }
        boolean flag = true;

        for (ClashMsgWithCaseInfo item : clashMsgWithCaseInfos) {
            if (item.getMsgSub().size() > 0) {
                flag = false;
                break;
            }
        }
        if (flag) {
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "操作成功");
            ajaxResult.put("data", "noClashInfo");
        } else {
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "操作成功");
            ajaxResult.put("data", clashMsgWithCaseInfos);
        }
        return ajaxResult;

    }


    @Override
    public AjaxResult ClashMsgBatch(List<ImpulseInformation> impulseInformation) {
        AjaxResult ajaxResult = new AjaxResult();
        List<ClashMsgWithCaseInfo> clashMsgWithCaseInfos = new ArrayList<>();
        for (ImpulseInformation item : impulseInformation) {
            ClashMsg clashMsg = new ClashMsg();
            ClashMsgWithCaseInfo clashMsgWithCaseInfo = new ClashMsgWithCaseInfo();

            item.setIsClient(1 - item.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(item);
            item.setIsClient(1 - item.getIsClient());
            CaseInformation caseItem = caseInformationMapper.selectById(item.getCaseId());  //查询当前案件信息【暂时不需要】

            clashMsgWithCaseInfo.setMsgMain("当前案件中：" + (item.getIsClient() == 0 ? "【委托方】->" : "【相对方】->") + item.getCaseName() + "[" + item.getCaseNo() + "]" + "与以下案件中的委托方/相对方有利冲嫌疑：");

            if (resList.size() > 0) {
                List<ClashMsgCaseInfoItem> list = new ArrayList<>();
                for (ImpulseInformation in : resList) {
                    CaseInformation ci = caseInformationMapper.selectById(in.getCaseId());
                    ClashMsgCaseInfoItem clashMsgCaseInfoItem = new ClashMsgCaseInfoItem();
                    clashMsgCaseInfoItem.setCaseId(ci.getId());
                    clashMsgCaseInfoItem.setMsg(ci.getCaseName() + "【" + ci.getCaseNo() + "】案件中的" + (in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "[" + in.getCaseNo() + "]" + "有疑似利冲信息");
                    list.add(clashMsgCaseInfoItem);
                }
                clashMsgWithCaseInfo.setMsgSub(list);
                clashMsgWithCaseInfos.add(clashMsgWithCaseInfo);
            }
        }

        System.out.println(impulseInformation);

        boolean flag = true;

        for (ClashMsgWithCaseInfo item : clashMsgWithCaseInfos) {
            if (item.getMsgSub().size() > 0) {
                flag = false;
                break;
            }
        }
        if (flag) {
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "操作成功");
            ajaxResult.put("data", "noClashInfo");
        } else {
            ajaxResult.put("code", 200);
            ajaxResult.put("msg", "操作成功");
            ajaxResult.put("data", clashMsgWithCaseInfos);
        }
        return ajaxResult;
    }


    /*@Override
    public Object ClashMsgBatch(List<ImpulseInformation> impulseInformation) {
        List<ClashMsg> clashMsgList = new ArrayList<>();
        for (ImpulseInformation item : impulseInformation) {
            ClashMsg clashMsg = new ClashMsg();
            item.setIsClient(1 - item.getIsClient());
            List<ImpulseInformation> resList = impulseInformationMapper.retrievalConflict(item);
            item.setIsClient(1 - item.getIsClient());
            CaseInformation caseItem = caseInformationMapper.selectById(item.getCaseId());  //查询当前案件信息【暂时不需要】
            clashMsg.setMsgMain("当前案件中：" + (item.getIsClient() == 0 ? "【委托方】->" : "【相对方】->") + item.getCaseName() + "[" + item.getCaseNo() + "]" + "与以下案件中的委托方/相对方有利冲嫌疑：");
            ArrayList<String> strings = new ArrayList<>();
            for (ImpulseInformation in : resList) {
                CaseInformation ci = caseInformationMapper.selectById(in.getCaseId());
                strings.add(ci.getCaseName() + "【" + ci.getCaseNo() + "】案件中的" + (in.getIsClient() == 0 ? "【委托方】" : "【相对方】") + in.getCaseName() + "[" + in.getCaseNo() + "]" + "有疑似利冲信息");
            }
            clashMsg.setMsgSub(strings);
            clashMsgList.add(clashMsg);
        }

        System.out.println(impulseInformation);

        return clashMsgList;
    }*/

    @Override
    public ImpulseInformation getById(Long id) {
        return impulseInformationMapper.selectById(id);
    }

    @Override
    public Integer create(ImpulseInformation impulseInformation) {
        return impulseInformationMapper.insert(impulseInformation);
    }

    @Override
    public Integer delete(Long id) {
        return impulseInformationMapper.deleteById(id);
    }

    @Override
    public Integer update(ImpulseInformation impulseInformation) {
        return impulseInformationMapper.updateById(impulseInformation);
    }


}
